@@ -1,6 +1,9 @@
 #_____________________________________________________________#
 #### Private functions ####
 
+# Convenience operator like Ruby's || operator. Returns a if it is not null, otherwise b
+`%||%` <- function(a, b) if (!is.null(a)) a else b
+
 # Returns the label to be used to represent the specified diff
 getDiffLabel <- function(pwes) {
   label <- attr(pwes, "label")
@@ -113,8 +116,15 @@ getErrorBars <- function(es, groupIdx, groupMean, error.bars) {
 # Function to add text and tick marks to the x-axis, applying consistent spacing
 # etc.. Labels are positioned and aligned along the top of the bounding box, so
 # multi-line labels descend lower
-labelXAxis <- function(at, labels, tick) {
-  graphics::axis(1, tick = tick, at = at, labels = labels, padj = 1, mgp = c(3, 0, 0))
+#
+# @param params Added to help with future improvement: give users greater
+#   control over axis display
+labelXAxis <- function(at, labels, tick, params = list()) {
+  tick <- params[["tick"]] %||% tick
+  padj <- params[["padj"]] %||% 1
+  mgp <- params[["mgp"]] %||% c(3, 0, 0)
+  params[c("tick", "padj", "mgp")] <- NULL
+  do.call(graphics::axis, c(list(1, at = at, labels = labels, tick = tick, padj = padj, mgp = mgp), params))
 }
 
 
@@ -147,7 +157,7 @@ rangeOfDiff <- function(pwes, violin.trunc) {
 #   specified, no mapping is performed.
 plotEffectSize <- function(pwes, xo, centreY,
                            showViolin, violinCol, violin.fill, violin.width, violin.shape, violin.trunc,
-                           ef.size.col, ef.size.pch, ef.size.lty, ef.size.lwd, mapYFn = identity, xpd = FALSE) {
+                           ef.size.col, ef.size.pch, ef.size.cex, ef.size.lty, ef.size.lwd, mapYFn = identity, xpd = FALSE) {
   deltaY <- centreY - pwes$t0
   if (showViolin) {
     d <- stats::density(pwes$t, na.rm = TRUE)
@@ -168,7 +178,7 @@ plotEffectSize <- function(pwes, xo, centreY,
   }
 
   # Draw mean of effect size
-  graphics::points(xo, mapYFn(pwes$t0 + deltaY), pch = ef.size.pch, col = ef.size.col, cex = 1.5, xpd = xpd)
+  graphics::points(xo, mapYFn(pwes$t0 + deltaY), pch = ef.size.pch, col = ef.size.col, cex = ef.size.cex, xpd = xpd)
   # Confidence interval of effect size
   graphics::segments(xo, mapYFn(pwes$bca[4] + deltaY), xo, mapYFn(pwes$bca[5] + deltaY),
            col = ef.size.col, lty = ef.size.lty, lwd = ef.size.lwd, xpd = xpd)
@@ -186,9 +196,9 @@ buildMapYFn <- function(xc, yc) {
 }
 
 # Plot effect size to the right of the main plot. Only useful when showing a single effect size
-plotEffectSizesRight <- function(es, pwes, ef.size.col, ef.size.pch, ef.size.lty, ef.size.lwd,
+plotEffectSizesRight <- function(es, pwes, ef.size.col, ef.size.pch, ef.size.cex, ef.size.lty, ef.size.lwd,
                                  showViolin, violinCol, violin.fill, violin.width, violin.shape, violin.trunc,
-                                 ef.size.dx, axisLabel, ticksAt,
+                                 ef.size.dx, axisLabel, ticksAt, ylim,
                                  groupX, ef.size.line.col, ef.size.line.lty, ef.size.line.lwd) {
 
   # Get the means of the 2 groups
@@ -201,10 +211,13 @@ plotEffectSizesRight <- function(es, pwes, ef.size.col, ef.size.pch, ef.size.lty
   x <- length(es$groups) + 1 + ef.size.dx
 
   if (!is.function(es$effect.type) && es$effect.type == "mean") {
+    # When the effect type is mean, units for effect size (i.e. y-axis) are the
+    # same as the main plot y axis
+
     esRange <- range(c(0, pwes$t))
     plotEffectSize(pwes, x, y,
                    showViolin, violinCol, violin.fill, violin.width, violin.shape, violin.trunc,
-                   ef.size.col, ef.size.pch, ef.size.lty, ef.size.lwd)
+                   ef.size.col, ef.size.pch, ef.size.cex, ef.size.lty, ef.size.lwd)
 
     # Axis labels on right-hand
     labels <- names(ticksAt)
@@ -216,13 +229,18 @@ plotEffectSizesRight <- function(es, pwes, ef.size.col, ef.size.pch, ef.size.lty
     }
     graphics::axis(4, at = y2 + ticksAt, labels = labels)
   } else {
-    esRange <- range(c(0, pwes$t0))
+    # Any effect type other than mean and we have to map the y-axis to the main plot y-axis
+    if (is.null(ylim)) {
+      esRange <- range(c(0, pwes$t0))
+    } else {
+      esRange <- ylim
+    }
     ylim <- range(y, y2)
     # Function to map esRange to ylim
     mapY <- buildMapYFn(esRange, ylim)
     plotEffectSize(pwes, x, pwes$t0,
                    showViolin, violinCol, violin.fill, violin.width, violin.shape, violin.trunc,
-                   ef.size.col, ef.size.pch, ef.size.lty, ef.size.lwd, mapYFn = mapY)
+                   ef.size.col, ef.size.pch, ef.size.cex, ef.size.lty, ef.size.lwd, mapYFn = mapY)
 
     # Axis labels on right-hand
     labels <- names(ticksAt)
@@ -260,21 +278,26 @@ plotEffectSizesRight <- function(es, pwes, ef.size.col, ef.size.pch, ef.size.lty
 #   bottom of the main plot region and the top of the effect size plot region.
 # @param plotProportion Height of the effect size plot region as a proportion of
 #   the main plot region.
-plotEffectSizesBelow <- function(es, plotDiffs, ef.size.col, ef.size.pch, ef.size.lty, ef.size.lwd,
+plotEffectSizesBelow <- function(es, plotDiffs, ef.size.col, ef.size.pch, ef.size.cex, ef.size.lty, ef.size.lwd,
                                  showViolin, violinCol, violin.fill, violin.width, violin.shape, violin.trunc,
-                                 xlim, ef.size.dx, ef.size.label, ticksAt,
+                                 xlim, ef.size.dx, ef.size.label, ticksAt, ylim,
                                  ef.size.line.col, ef.size.line.lty, ef.size.line.lwd,
                                  group.dx,
                                  paddingTop, plotProportion) {
   groups <- es$groups
 
   # What will we plot?
-  # Range of effect sizes
-  ylimData <- range(sapply(plotDiffs, rangeOfDiff, violin.trunc), na.rm = TRUE)
-  # Ensure ylim includes 0
-  ylimData <- range(c(0, ylimData), na.rm = TRUE)
-  # Extend range the same way a normal plot does
-  ylim <- grDevices::extendrange(ylimData)
+  if (!is.null(ylim)) {
+    # ylimData is used to calculate where to put the ticks
+    ylimData <- ylim
+  } else {
+    # Range of effect sizes
+    ylimData <- range(sapply(plotDiffs, rangeOfDiff, violin.trunc), na.rm = TRUE)
+    # Ensure ylim includes 0
+    ylimData <- range(c(0, ylimData), na.rm = TRUE)
+    # Extend range the same way a normal plot does
+    ylim <- grDevices::extendrange(ylimData)
+  }
 
   # Vertical layout. The effect size is plotted into the bottom margin. Layout
   # calculations are performed as follows:
@@ -328,6 +351,7 @@ plotEffectSizesBelow <- function(es, plotDiffs, ef.size.col, ef.size.pch, ef.siz
   # Recycle parameters
   ef.size.col <- rep_len(ef.size.col, length(plotDiffs))
   ef.size.pch <- rep_len(ef.size.pch, length(plotDiffs))
+  ef.size.cex <- rep_len(ef.size.cex, length(plotDiffs))
   ef.size.lty <- rep_len(ef.size.lty, length(plotDiffs))
   ef.size.lwd <- rep_len(ef.size.lwd, length(plotDiffs))
 
@@ -339,16 +363,13 @@ plotEffectSizesBelow <- function(es, plotDiffs, ef.size.col, ef.size.pch, ef.siz
       # Add default group dx for the displayed group to effect size dx for the contrast
       x <- gid1 + group.dx[gid1] + ef.size.dx[i]
       plotEffectSize(pwes, x, pwes$t0, showViolin, violinCol, violin.fill, violin.width, violin.shape, violin.trunc,
-                     ef.size.col[i], ef.size.pch[i], ef.size.lty[i], ef.size.lwd[i], mapY, xpd = TRUE)
+                     ef.size.col[i], ef.size.pch[i], ef.size.cex[i], ef.size.lty[i], ef.size.lwd[i], mapY, xpd = TRUE)
       # Label this difference
       graphics::text(x, mapY(ylim[1]), getDiffLabel(pwes), xpd = TRUE, pos = 1)
     }
   }
 }
 
-# Convenience operator like Ruby's || operator. Returns a if it is not null, otherwise b
-`%||%` <- function(a, b) if (!is.null(a)) a else b
-
 #_____________________________________________________________#
 #### Public functions ####
 
@@ -461,7 +482,7 @@ DurgaTransparent <-  function(colour, transparency, relative = FALSE) {
 #'   \code{group.dx = c(0.1, -0.1)} will group into pairs. Individual components
 #'   can be shifted independently using the appropriate \code{*.dx} parameters.
 #' @param group.colour Colours to use for each group. Either an
-#'   \code{\link{RColorBrewer}} palette name or a vector of colours.
+#'   \code{\link[RColorBrewer:RColorBrewer]{RColorBrewer}} palette name or a vector of colours.
 #'
 #' @param points If not \code{FALSE}, points are plotted. If \code{TRUE}, points
 #'   are displayed with a default colour (which is the group colour with 40%
@@ -562,8 +583,7 @@ DurgaTransparent <-  function(colour, transparency, relative = FALSE) {
 #' @param paired.lwd Line width for pair lines.
 #'
 #' @param ef.size If not \code{FALSE}, effect sizes are plotted. May be
-#'   \code{TRUE} or
-#'   a colour.
+#'   \code{TRUE} or a colour.
 #' @param ef.size.position Effect sizes are plotted to the right of the main
 #'   plot if there is only one effect size to plot and \code{ef.size.position !=
 #'   "below"}. If the effect size is drawn to the right, you will need to
@@ -579,6 +599,7 @@ DurgaTransparent <-  function(colour, transparency, relative = FALSE) {
 #' @param ef.size.violin.fill Colour used to fill effect size violins. Default
 #'   is a transparent version of \code{ef.size.violin}.
 #' @param ef.size.pch Symbol to represent mean effect size.
+#' @param ef.size.cex Mean effect size symbol magnification (default is 1.5).
 #' @param ef.size.lty Line style for the effect size error bar.
 #' @param ef.size.lwd Line weight for the effect size error bar.
 #' @param ef.size.dx Horizontal shift to be applied to each contrast/effect
@@ -591,6 +612,7 @@ DurgaTransparent <-  function(colour, transparency, relative = FALSE) {
 #'   negative effect" = -0.5, "Small negative effect" = -0.2, "No effect" = 0,
 #'   "Small positive effect" = 0.2, "Medium positive effect" = 0.5, "Large
 #'   positive effect" = 0.8)}
+#' @param ef.size.ylim If specified, overrides the y-axis limits of the effect size plot.
 #' @param ef.size.top.pad Gap (in units of default character height scaled by
 #'   \code{cex}) between the bottom of the main plot region and the top of the
 #'   effect size plot region. Only applies when effect size is positioned below.
@@ -738,9 +760,11 @@ DurgaPlot <- function(es,
                     ef.size.violin.shape = c("right-half", "left-half", "full"),
                     ef.size.violin.trunc = TRUE,
                     ef.size.pch = 17,
+                    ef.size.cex = 1.5,
                     ef.size.lty = 1,
                     ef.size.lwd = 2,
                     ef.size.ticks = NULL,
+                    ef.size.ylim = NULL,
                     ef.size.label = es$effect.name,
                     ef.size.dx = 0,
                     ef.size.adj.margin = TRUE,
@@ -856,6 +880,10 @@ DurgaPlot <- function(es,
     # Can't show more than one effect size to the right
     ef.size.position <- "below"
   }
+  # ef.size.ylim can't be applied when ef size is on the right and effect type is mean, because in that case effect size y-axis must match the main plot y-axis
+  if (!is.null(ef.size.ylim) && ef.size.position == "right" && es$effect.type == "mean") {
+    stop("Argument ef.size.ylim cannot be specified when the ef.size.position is \"right\" and effect type is \"mean\"")
+  }
 
 
   # Calculate plot limits
@@ -1087,7 +1115,7 @@ DurgaPlot <- function(es,
   # Draw lines between paired points
   if (.show(paired)) {
     if (!es$paired.data)
-      stop("To plot paired lines, data must be, i.e. id.col specified to DurgaDiffs")
+      stop("To plot paired lines, data must be paired, i.e. id.col specified to DurgaDiffs")
     col <- .boolToDef(paired, DurgaTransparent("grey20", 0.7))
     # Display all contrasts, which can get very ugly if there's more than one
     for (i in seq_len(length(plotDiffs))) {
@@ -1162,14 +1190,14 @@ DurgaPlot <- function(es,
 
     if (ef.size.position == "right") {
       lineStartAt <- seq_len(nGroups) + ef.size.mean.line.dx
-      plotEffectSizesRight(es, plotDiffs[[1]], ef.size.col, ef.size.pch, ef.size.lty, ef.size.lwd,
+      plotEffectSizesRight(es, plotDiffs[[1]], ef.size.col, ef.size.pch, ef.size.cex, ef.size.lty, ef.size.lwd,
                            .show(ef.size.violin), violinCol, violinFill, violin.width, ef.size.violin.shape, ef.size.violin.trunc,
-                           ef.size.dx, ef.size.label, ef.size.ticks,
+                           ef.size.dx, ef.size.label, ef.size.ticks, ef.size.ylim,
                            lineStartAt, ef.size.line.col, ef.size.line.lty, ef.size.line.lwd)
     } else if (ef.size.position == "below") {
-      plotEffectSizesBelow(es, plotDiffs, ef.size.col, ef.size.pch, ef.size.lty, ef.size.lwd,
+      plotEffectSizesBelow(es, plotDiffs, ef.size.col, ef.size.pch, ef.size.cex, ef.size.lty, ef.size.lwd,
                            .show(ef.size.violin), violinCol, violinFill, violin.width, ef.size.violin.shape, ef.size.violin.trunc,
-                           xlim, ef.size.dx, ef.size.label, ef.size.ticks,
+                           xlim, ef.size.dx, ef.size.label, ef.size.ticks, ef.size.ylim,
                            ef.size.line.col, ef.size.line.lty, ef.size.line.lwd, group.dx,
                            ef.size.top.pad, ef.size.height)
     }
